  -- =========================================================================
  -- Class TStack
  --
  -- This class defines a stack.
  -- =========================================================================
  do
  -- -------------------------------------------------------------------------
  -- Method IsEmpty
  --
  -- This method returns true when the stack is empty or false otherwise.
  -- -------------------------------------------------------------------------
    local function IsEmpty(self)
      return (#self.m_vecItems == 0)
    end

  -- -------------------------------------------------------------------------
  -- Method Peek
  --
  -- This method returns the most recently stacked item without popping it off
  -- -------------------------------------------------------------------------
    local function Peek(self)
      local Item 
	  
      if (0 < #self.m_vecItems) then
        Item = self.m_vecItems[#self.m_vecItems]
      end
	  
      return Item
    end  
  
  -- -------------------------------------------------------------------------
  -- Method Pop
  --
  -- This method returns the most recently stacked item.
  -- -------------------------------------------------------------------------
    local function Pop(self)
      local Item 
	  
      if (0 < #self.m_vecItems) then
        Item = self.m_vecItems[#self.m_vecItems]
        table.remove(self.m_vecItems)
      end
	  
      return Item
    end  

  -- -------------------------------------------------------------------------
  -- Method Push
  --
  -- This method places an item on the stack.
  -- -------------------------------------------------------------------------
    local function Push(self, a_Item)
      table.insert(self.m_vecItems, a_Item)   
    end

  -- -------------------------------------------------------------------------
  -- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
     local function GetType()
       return "TStack"
     end

  -- -------------------------------------------------------------------------
  -- Method Size
  --
  -- This method returns the number of items on the stack.
  -- -------------------------------------------------------------------------
    local function Size(self)
      local numItemCount
	  
	  numItemCount = #self.m_vecItems
	  
      return numItemCount
    end  
	 
  -- -------------------------------------------------------------------------
  -- Method new_TStack
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
    function new_TStack()
      local obj =
      {
        GetType    = GetType,
        IsEmpty    = IsEmpty,
		Peek       = Peek,
        Pop        = Pop,
        Push       = Push,
		Size       = Size,
        m_vecItems = {},
      }
	  
      return obj
    end
  end


do -- Class TModelCalculator
  -- -------------------------------------------------------------------------
  -- Method IsLeapYear
  --
  -- This method returns true when the given year is a leap year.
  -- If not, it returns false.
  -- Reference: Jean Meeus, _Astronomical Algorithms_, second edition, 1998, Willmann-Bell, Inc.
  -- -------------------------------------------------------------------------
  local function IsLeapYear(a_numYears, gregorian)
    local numYears
    local isLeap
    
    if nil == gregorian then 
      gregorian = true 
    end
    
    numYears = math.floor(a_numYears)
	  
    if gregorian then
        isLeap = ((numYears % 4 == 0) and ((numYears % 100 ~= 0) or (numYears % 400 == 0)))
    else
        isLeap = (numYears % 4 == 0)
    end
    
    return isLeap
  end  


  local function IsDate(a_strValue)
    local numReturnValue = true
	
    if (string.match(a_strValue, "%d%d%d%d/%d%d%d/%d%d:%d%d:%d%dZ")) then
	    local numYears
	    local numDaysOfYear
	    local numHours
	    local numMinutes
	    local numSeconds
	  
	    numYears      = tonumber(string.sub(a_strValue,  1,  4))
	    numDaysOfYear = tonumber(string.sub(a_strValue,  6,  8))
	    numHours      = tonumber(string.sub(a_strValue, 10, 11))
	    numMinutes    = tonumber(string.sub(a_strValue, 13, 14))
	    numSeconds    = tonumber(string.sub(a_strValue, 16, 17))
	  
	    if (IsLeapYear(numYears)) then
	      if (366<numDaysOfYear) then
		      numReturnValue = false
		    end
	    else
	      if (365<numDaysOfYear) then
		      numReturnValue = false
		    end
	    end
	  
	    if (59<numHours)   then numReturnValue = false end
	    if (59<numMinutes) then numReturnValue = false end
	    if (59<numSeconds) then numReturnValue = false end
	  else
	    numReturnValue = false
	  end
    
    return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method SetValue
  --
  -- This method pushes a value onto the stack for later evaluation.
  -- -------------------------------------------------------------------------
  local function SetValue(self)
    local strValue
    local numValue

    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)
	if (nil==string.find(strValue, 'Z')) then -- Must be a number
	  numValue = tonumber(strValue)
      self.tblStack:Push(numValue)
	else -- Must be a date
	  self.tblStack:Push(strValue)
	end
	
	self.strValue = ""
  end

  local function cal_to_day_of_year(yr, mo, a_numDays, gregorian)
	  local K
    local numDays
    
    if gregorian == nil then 
      gregorian = true 
    end
    
    if IsLeapYear(yr, gregorian) then 
        K = 1
    else
        K = 2
    end
	
	  numDays = math.floor(a_numDays)
    return math.floor(275 * mo / 9.0) - (K * math.floor((mo + 9) / 12.0)) + numDays - 30
  end  
  
  local function day_of_year_to_cal(a_numYears, a_numDaysOfYear, gregorian)
    local K
    local mo
    local dy
	
    if nil == gregorian then 
      gregorian = true 
    end
	
    if IsLeapYear(a_numYears, gregorian) then
      K = 1
    else
      K = 2
    end
	
    if (32 > a_numDaysOfYear) then
        mo = 1
    else
        mo = math.floor(9 * (K + a_numDaysOfYear) / 275.0 + 0.98)
    end
    
	  dy = math.floor(a_numDaysOfYear - math.floor(275 * mo / 9.0) + K * math.floor((mo + 9) / 12.0) + 30)
    return mo, dy
  end
  
  -- -------------------------------------------------------------------------
  -- Method computeJulianDate
  --
  -- This method returns the Julian Date given a calendar date.
  -- Reference: Jean Meeus, _Astronomical Algorithms_, second edition, 1998, Willmann-Bell, Inc.
  -- -------------------------------------------------------------------------
  local function computeJulianDate(a_strCalendarDate, gregorian)
    local numJD
    local numYears
    local numDaysOfYear
    local numMonths
    local numDays
    local numHours
    local numMinutes
    local numSeconds
    local numA
    local numB
    local strYears
    local strDaysOfYear
    local strHours
    local strMinutes
    local strSeconds

	  strYears      = string.sub(a_strCalendarDate,  1,  4)
    strDaysOfYear = string.sub(a_strCalendarDate,  6,  8)
		strHours      = string.sub(a_strCalendarDate, 10, 11)
		strMinutes    = string.sub(a_strCalendarDate, 13, 14)
		strSeconds    = string.sub(a_strCalendarDate, 16, 17)
		numYears      = tonumber(strYears)
		numDaysOfYear = tonumber(strDaysOfYear)
		numHours      = tonumber(strHours)
		numMinutes    = tonumber(strMinutes)
		numSeconds    = tonumber(strSeconds)
    
    numMonths, numDays = day_of_year_to_cal(numYears, numDaysOfYear, gregorian)
    numDays            = numDays + (numHours / 24) + (numMinutes / 1440) + (numSeconds / 86400)
    numMonths          = numMonths or 1
	  numDays            = numDays or 1
    
	  if nil == gregorian then 
      gregorian = true 
    end
    
    if numMonths <= 2 then
        numYears = numYears - 1
        numMonths = numMonths + 12
	  end
    
    if gregorian then
        numA = math.floor(numYears / 100)
        numB = 2 - numA + math.floor(numA / 4)
    else
        numB = 0
	  end
    
    numJD = math.floor(365.25 * (numYears + 4716)) + math.floor(30.6001 * (numMonths + 1)) + numDays + numB - 1524.5
    return numJD
  end

  local function jd_to_cal(jd, gregorian)
	  if nil == gregorian then 
      gregorian = true 
    end
    
    local F, Z, A, mo, yr
	
	  Z, F = math.modf(jd + 0.5)
    
    if gregorian then
        local alpha = math.floor((Z - 1867216.25) / 36524.25)
        A = Z + 1 + alpha - math.floor(alpha / 4)
    else
        A = Z
	  end
	
    local B = A + 1524
    local C = math.floor((B - 122.1) / 365.25)
    local D = math.floor(365.25 * C)
    local E = math.floor((B - D) / 30.6001)
    local day = B - D - math.floor(30.6001 * E) + F
    
    if E < 14 then
        mo = E - 1
    else
        mo = E - 13
	  end
	
	  if mo > 2 then
        yr = C - 4716
    else
        yr = C - 4715
    end

    local numDaysOfYear
    local numDays
    local numHours
    local numFractionalDays
    local numFractionalHours
    local numFractionalMinutes
    local numFractionalSeconds
    local numMinutes
    local numSeconds
  
    numDaysOfYear                    = cal_to_day_of_year(yr, mo, day, gregorian)    
    numDays,  numFractionalDays      = math.modf(day)
    numHours, numFractionalHours     = math.modf(24.0 * numFractionalDays)
    numMinutes, numFractionalMinutes = math.modf(60.0 * numFractionalHours)
    numSeconds, numFractionalSeconds = math.modf(60.0 * numFractionalMinutes)   

    if (0.5 < numFractionalSeconds) then
      numSeconds = 1 + numSeconds
      
      if (60 == numSeconds) then
        numMinutes = 1 + numMinutes
        numSeconds = 0
      end
      
      if (60 == numMinutes) then
        numHours = 1 + numHours
        numMinutes = 0
      end
      
      if (24 == numHours) then
        numDaysOfYear = 1 + numDaysOfYear
        numHours = 0
      end
      
      if (IsLeapYear(yr)) then
        if (366 < numDaysOfYear) then
          yr = 1 + yr
          numDaysOfYear = 1
        end
      else
        if (365 < numDaysOfYear) then
          yr = 1 + yr
          numDaysOfYear = 1
        end
      end
    end
    
	  return yr, numDaysOfYear, numHours, numMinutes, numSeconds
  end
    
  local function jd_to_day_of_week(jd)
    local i = jd + 1.5
    return math.floor(i) % 7
  end

  
  -- -------------------------------------------------------------------------
  -- Method AppendDecimalPoint
  --
  -- This method appends a decimal point to the string when there is not 
  -- already a decimal point in the string and it would not be the 15th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDecimalPoint(self)
    local strValue
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)

	-- Note: '.' is a wildcard character.  
	--       It should be escaped with a % to actually search for a decimal point.
	if (nil==string.find(strValue, '%.')) then
	  local numStringLength
	  
	  numStringLength = string.len(strValue)
	  
	  if (15 > numStringLength) then
	    if (0 == numStringLength) then
		  strValue = "0."
		else
	      strValue = string.format("%s.", strValue)
		end
	  end
    end
	
	self.strValue = strValue
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit0
  --
  -- This method appends a zero to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit0(self)
    local strValue
	local numPositionZero
	local numPositionColon
	local numPositionSlash    
	local numStringLength	
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	
	-- append 0 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s0", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s0", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s0/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s0", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s0", strValue)
		--print(numStringLength)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s0", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/0", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s0/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s0", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s0", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s0:", strValue)
		  else
		    strValue = string.format("%s%02s:0", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s0", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s0", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s0:", strValue)
		  else
		    strValue = string.format("%s%02s:0", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s0", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s0", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s0Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 0 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s0", strValue)
	  end
    end	
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit1
  --
  -- This method appends a one to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit1(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	
	-- append 1 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s1", strValue)
		end
		if (2==numStringLength) then
		  strValue = string.format("%s1", strValue)
		end
		if (3==numStringLength) then
          strValue = string.format("%s1/", strValue)
        end	
		if (5==numStringLength) then
		  strValue = string.format("%s1", strValue)
		end
		if (6==numStringLength) then
		  strValue = string.format("%s1", strValue)
		end		
        if (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strTemp
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s1", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
          strTemp = string.sub(strValue, 1, 5)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/1", strTemp, string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s1/", strValue)
		  end
		  strTemp = string.sub(strValue, 1, 5)
        end	
		if (9==numStringLength) then
		  strValue = string.format("%s1", strValue)
		end
        if (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  local strTemp
		  
		  strTemp = string.sub(strValue, 1, 9)
		  strHours = string.format("%s1", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s1:", strValue)
		  else
		    strValue = string.format("%s%02s:1", strTemp, string.sub(strValue, 10, 10))
		  end
		end
		if (12==numStringLength) then
		  strValue = string.format("%s1", strValue)
		end
        if (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  local strTemp
		  
		  strTemp = string.sub(strValue, 1, 12)
		  strMinutes = string.format("%s1", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s1:", strValue)
		  else
		    strValue = string.format("%s%02s:1", strTemp, string.sub(strValue, 13, 13))
		  end
        end		  
		if (15==numStringLength) then
		  strValue = string.format("%s1", strValue)
		end
		if (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  local strTemp
		  
		  strTemp = string.sub(strValue, 1, 15)
		  strSeconds = string.format("%s1", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s1Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", strTemp, string.sub(strValue, 16, 16))
		  end
		end
		
      end
	-- append 1 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s1", strValue)
	  else
	    strValue = "1"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit2
  --
  -- This method appends a two to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit2(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	
	-- append 2 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s2", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s2", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s2/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s2", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s2", strValue)
		--print(numStringLength)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s2", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/2", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s2/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s2", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s2", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s2:", strValue)
		  else
		    strValue = string.format("%s%02s:2", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s2", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s2", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s2:", strValue)
		  else
		    strValue = string.format("%s%02s:2", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s2", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s2", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s2Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 2 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s2", strValue)
	  else
	    strValue = "2"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit3
  --
  -- This method appends a three to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit3(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	
	-- append 3 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s3", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s3", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s3/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s3", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s3", strValue)
		--print(numStringLength)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s3", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/3", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s3/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s3", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s3", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s3:", strValue)
		  else
		    strValue = string.format("%s%02s:3", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s3", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s3", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s3:", strValue)
		  else
		    strValue = string.format("%s%02s:3", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s3", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s3", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s3Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 3 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s3", strValue)
	  else
	    strValue = "3"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit4
  --
  -- This method appends a four to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit4(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	
	-- append 4 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s4", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s4", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s4/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s4", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s4", strValue)
		--print(numStringLength)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s4", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/4", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s4/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s4", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s4", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s4:", strValue)
		  else
		    strValue = string.format("%s%02s:4", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s4", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s4", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s4:", strValue)
		  else
		    strValue = string.format("%s%02s:4", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s4", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s4", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s4Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 4 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s4", strValue)
	  else
	    strValue = "4"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit5
  --
  -- This method appends a five to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit5(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	
	-- append 5 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s5", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s5", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s5/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s5", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s5", strValue)
		--print(numStringLength)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s5", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/5", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s5/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s5", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s5", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s5:", strValue)
		  else
		    strValue = string.format("%s%02s:5", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s5", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s5", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s5:", strValue)
		  else
		    strValue = string.format("%s%02s:5", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s5", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s5", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s5Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 5 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s5", strValue)
	  else
	    strValue = "5"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit6
  --
  -- This method appends a six to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit6(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	print(numStringLength)
	
	-- append 6 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s6", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s6", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s6/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s6", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s6", strValue)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s6", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/06:", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s6/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s6", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s6", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s6:", strValue)
		  else
		    strValue = string.format("%s%02s:06:", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s6", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s6", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s6:", strValue)
		  else
		    strValue = string.format("%s%02s:06Z", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s6", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s6", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s6Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 6 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s6", strValue)
	  else
	    strValue = "6"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit7
  --
  -- This method appends a seven to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------  
  local function AppendDigit7(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	print(numStringLength)
	
	-- append 7 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s7", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s7", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s7/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s7", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s7", strValue)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s7", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/07:", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s7/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s7", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s7", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s7:", strValue)
		  else
		    strValue = string.format("%s%02s:07:", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s7", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s7", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s7:", strValue)
		  else
		    strValue = string.format("%s%02s:07Z", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s7", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s7", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s7Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 7 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s7", strValue)
	  else
	    strValue = "7"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit8
  --
  -- This method appends an eight to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit8(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	print(numStringLength)
	
	-- append 8 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s8", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s8", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s8/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s8", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s8", strValue)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s8", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/08:", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s8/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s8", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s8", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s8:", strValue)
		  else
		    strValue = string.format("%s%02s:08:", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s8", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s8", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s8:", strValue)
		  else
		    strValue = string.format("%s%02s:08Z", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s8", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s8", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s8Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 8 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s8", strValue)
	  else
	    strValue = "8"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method AppendDigit9
  --
  -- This method appends a two to the string when it would not be the 16th
  -- character in the string.
  -- -------------------------------------------------------------------------
  local function AppendDigit9(self)
    local strValue 
	local numPositionZero
	local numPositionColon
	local numPositionSlash
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionZero = string.find(strValue, '0')
	numPositionColon = string.find(strValue, ':')
	numPositionSlash = string.find(strValue, '/')
	numStringLength = string.len(strValue)
	print(numStringLength)
	
	-- append 9 to a date/timestamp
	if (1==numPositionZero) or (5==numPositionSlash) then
      if (17 > numStringLength) then
	    if (1==numStringLength) then
		  strValue = string.format("%s9", strValue)
		elseif (2==numStringLength) then
		  strValue = string.format("%s9", strValue)
		elseif (3==numStringLength) then
          strValue = string.format("%s9/", strValue)
		elseif (5==numStringLength) then
		  strValue = string.format("%s9", strValue)
		elseif (6==numStringLength) then
		  strValue = string.format("%s9", strValue)
        elseif (7==numStringLength) then -- Note: keep the day of year valid
          local numDayOfYear
		  local numMaxDaysOfYear
		  local numYear
		  local strDayOfYear
		  local strYear
		  
		  strYear = string.sub(strValue, 1, 4)
		  numYear=tonumber(strYear)
		  
		  if (IsLeapYear(numYear)) then
		    numMaxDaysOfYear = 366
		  else
		    numMaxDaysOfYear = 365
		  end
		  
		  strDayOfYear = string.format("%s9", string.sub(strValue, 6, 7))
		  numDayOfYear = tonumber(strDayOfYear)
		  
		  if (numDayOfYear > numMaxDaysOfYear) then
		    strValue = string.format("%s%03s/09:", string.sub(strValue, 1, 5), string.sub(strValue, 6, 7))
		  else
            strValue = string.format("%s9/", strValue)
		  end
		elseif (9==numStringLength) then
		  strValue = string.format("%s9", strValue)
        elseif (10==numStringLength) then -- Note: Keep the hours valid
		  local numHours
		  local strHours
		  
		  strHours = string.format("%s9", string.sub(strValue, 10, 11))
		  numHours = tonumber(strHours)
		  
		  if (60 > numHours) then
		    strValue = string.format("%s9:", strValue)
		  else
		    strValue = string.format("%s%02s:09:", string.sub(strValue, 1, 9), string.sub(strValue, 10, 10))
		  end
		elseif (12==numStringLength) then
		  strValue = string.format("%s9", strValue)
        elseif (13==numStringLength) then -- Note: Keep the minutes valid
		  local numMinutes
		  local strMinutes
		  
		  strMinutes = string.format("%s9", string.sub(strValue, 13, 14))
		  numMinutes = tonumber(strMinutes)
		  
		  if (60 > numMinutes) then
		    strValue = string.format("%s9:", strValue)
		  else
		    strValue = string.format("%s%02s:09Z", string.sub(strValue, 1, 12), string.sub(strValue, 13, 13))
		  end
		elseif (15==numStringLength) then
		  strValue = string.format("%s9", strValue)
		elseif (16==numStringLength) then -- Note: Keep the seconds valid
		  local numSeconds
		  local strSeconds
		  
		  strSeconds = string.format("%s9", string.sub(strValue, 16, 17))
		  numSeconds = tonumber(strSeconds)
		  
		  if (60 > numSeconds) then
		    strValue = string.format("%s9Z", strValue)
		  else
		    strValue = string.format("%s%02sZ", string.sub(strValue, 1, 15), string.sub(strValue, 16, 16))
		  end
		end		
      end
	-- append 9 to a number
	elseif (15 > numStringLength) then
	  if ("0"~=strValue) then -- prevent leading zeros
        strValue = string.format("%s9", strValue)
	  else
	    strValue = "9"
	  end
    end
	
	if (16==numStringLength) then -- last digit of date/timestamp
	  self.strValue = strValue
	  SetValue(self)
	else	
	  self.strValue = strValue
	end
	
    return strValue
  end

  -- -------------------------------------------------------------------------
  -- Method AppendCharSlash
  --
  -- This method appends a slash to the string
  -- -------------------------------------------------------------------------
  local function AppendCharSlash(self)
    local strValue 
	local numPositionSlash
	local numPositionSlash2
    local numStringLength
	local numStringLengthMax
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionSlash = string.find(strValue, '/')
	numPositionSlash2 = string.find(strValue, '/', 6)
	numStringLength = string.len(strValue)
	
	  if (nil~=strValue) then 
	    if ((5==numStringLength) and (5==numPositionSlash)) then
          -- print(strValue)
          strValue = string.format("%05s000/", strValue)
		else
		  if (nil==numPositionSlash2) then
		    if (5 < numStringLength) then
			  local strYear
		      local strDayOfYear
			
			  strYear = string.sub(strValue, 1, 4)
			  strDayOfYear = string.sub(strValue, 6)
			  strValue = string.format("%04s/%03s/", strYear, strDayOfYear)
			else
              strValue = string.format("%04s/", strValue)
			end
		  end
        end
      else
        strValue = "0000/"
      end
	
	self.strValue = strValue
    return self.strValue
  end

  -- -------------------------------------------------------------------------
  -- Method AppendCharColon
  --
  -- This method appends a colon to the string
  -- -------------------------------------------------------------------------
  local function AppendCharColon(self)
    local strValue 
	local numPositionSlash
	local numPositionSlash2
	local numPositionColon
	local numPositionColon2
    local numStringLength
	local numStringLengthMax
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numPositionSlash = string.find(strValue, '/')
	numPositionSlash2 = string.find(strValue, '/', 6)
	numPositionColon = string.find(strValue, ':')
	numPositionColon2 = string.find(strValue, ':', 13)
	numStringLength = string.len(strValue)
	
	  if (nil~=strValue) then
	    if (nil==numPositionSlash) then
		  AppendCharSlash(self)
          strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	      numPositionSlash = string.find(strValue, '/')
	      numPositionSlash2 = string.find(strValue, '/', 6)
	      numStringLength = string.len(strValue)
		end
		
		if (nil==numPositionSlash2) then
		  AppendCharSlash(self)
          strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	      numPositionSlash = string.find(strValue, '/')
	      numPositionSlash2 = string.find(strValue, '/', 6)
	      numStringLength = string.len(strValue)
		  strValue = string.format("%s00", strValue)
		end
		
		if (12>numStringLength) and (nil==numPositionColon) then
		  local strHour
		  local strTemp
		  
		  strTemp = string.sub(strValue, 1, 9)
		  strHour = string.sub(strValue, 10)
		  strValue = string.format("%s%02s:", strTemp, strHour)
		end

        if (12==numStringLength) and (12==numPositionColon) then
		  strValue = string.format("%s00:", strValue)
		else
		  if (nil==numPositionColon2) then
		    if (12<numStringLength) then
			  local strMin
			  local strTemp
			  
			  strTemp = string.sub(strValue, 1, 12)
			  strMin = string.sub(strValue, 13)
			  strValue = string.format("%s%02s:", strTemp, strMin)
			end
		  end
		end
		
	  else
	    strValue = "0000/000/00:"
	  end
	
	self.strValue = strValue
    return self.strValue
  end

  -- -------------------------------------------------------------------------
  -- Method AppendCharZ
  --
  -- This method appends a Z to the string to indicate Zulu time (UTC+0)
  -- -------------------------------------------------------------------------
  local function AppendCharZ(self)
    local strValue 
    local numStringLength
	
	AppendCharSlash(self)
	AppendCharSlash(self)
	AppendCharColon(self)
	AppendCharColon(self)
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)

	if (15==numStringLength) then
	  strValue = AppendDigit0(self)
	  strValue = AppendCharZ(self)
	else
	  strValue = AppendDigit0(self)
	end		
	
    return strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method ToggleSign
  --
  -- This method toggles the sign of the number.
  -- -------------------------------------------------------------------------
  local function ToggleSign(self)
    local strValue
	local strValueToggled
    local numStringLength
	
    strValue = string.gsub(self.strValue,"^%s*(.-)%s*$", "%1") -- trim(self.strValue)	
	numStringLength = string.len(strValue)
	
	if (0 < numStringLength) then 
	  if (1==string.find(strValue, '-')) then
	    strValueToggled = string.sub(strValue, 2, numStringLength)
	  else
	    strValueToggled = string.format("-%s", strValue)
	  end
	else
	  strValueToggled = "-"
	end
		
	self.strValue = strValueToggled
    return self.strValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetPlusSign
  --
  -- This method returns the plus sign.
  -- -------------------------------------------------------------------------
  local function GetPlusSign()
    return "+"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetMinusSign
  --
  -- This method returns the minus sign.
  -- -------------------------------------------------------------------------
  local function GetMinusSign()
    return "-"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetMultiplySign
  --
  -- This method returns the sign for multiplication.
  -- -------------------------------------------------------------------------
  local function GetMultiplySign()
    return "*"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetDivideSign
  --
  -- This method returns the sign for division.
  -- -------------------------------------------------------------------------
  local function GetDivideSign()
    return "/"
  end

  -- -------------------------------------------------------------------------
  -- Method GetPowerSign
  --
  -- This method returns the sign for power.
  -- -------------------------------------------------------------------------
  local function GetPowerSign()
    return "^"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetPlusMinus
  --
  -- This method returns the sign for plus or minus.
  -- -------------------------------------------------------------------------
  local function GetPlusMinus()
    return "±"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForAbsoluteValue
  --
  -- This method returns the text for absolute value.
  -- -------------------------------------------------------------------------
  local function GetTextForAbsoluteValue()
    return "math.abs"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForArcCosine
  --
  -- This method returns the text for arccosine.
  -- -------------------------------------------------------------------------
  local function GetTextForArcCosine()
    return "math.acos"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForArcSine
  --
  -- This method returns the text for arccosine.
  -- -------------------------------------------------------------------------
  local function GetTextForArcSine()
    return "math.asin"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForArcTangent
  --
  -- This method returns the text for arctangent
  -- -------------------------------------------------------------------------
  local function GetTextForArcTangent()
    return "math.atan"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForArcTangent2
  --
  -- This method returns the text for arctangent2
  -- -------------------------------------------------------------------------
  local function GetTextForArcTangent2()
    return "math.atan2"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForCeiling
  --
  -- This method returns the text for ceiling.
  -- -------------------------------------------------------------------------
  local function GetTextForCeiling()
    return "math.ceil"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForCosine
  --
  -- This method returns the text for cosine.
  -- -------------------------------------------------------------------------
  local function GetTextForCosine()
    return "math.cos"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForHyperbolicCosine
  --
  -- This method returns the text for hyperbolic cosine.
  -- -------------------------------------------------------------------------
  local function GetTextForHyperbolicCosine()
    return "math.cosh"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForDegrees
  --
  -- This method returns the text for degrees.
  -- -------------------------------------------------------------------------
  local function GetTextForDegrees()
    return "math.deg"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForExp
  --
  -- This method returns the text for e.
  -- -------------------------------------------------------------------------
  local function GetTextForExp()
    return "math.exp"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForFloor
  --
  -- This method returns the text for floor.
  -- -------------------------------------------------------------------------
  local function GetTextForFloor()
    return "math.floor"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForFMod
  --
  -- This method returns the text for fmod.
  -- -------------------------------------------------------------------------
  local function GetTextForFMod()
    return "math.fmod"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForNaturalLogarithm
  --
  -- This method returns the text for natural logarithm.
  -- -------------------------------------------------------------------------
  local function GetTextForNaturalLogarithm()
    return "math.log"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForBase10Logarithm
  --
  -- This method returns the text for base 10 logarithm.
  -- -------------------------------------------------------------------------
  local function GetTextForBase10Logarithm()
    return "math.log10"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForMaximum
  --
  -- This method returns the text for maximum.
  -- -------------------------------------------------------------------------
  local function GetTextForMaximum()
    return "math.max"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForMinimum
  --
  -- This method returns the text for minimum.
  -- -------------------------------------------------------------------------
  local function GetTextForMinimum()
    return "math.min"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForPi
  --
  -- This method returns the text for pi.
  -- -------------------------------------------------------------------------
  local function GetTextForPi()
    return "math.pi"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForPower
  --
  -- This method returns the text for power.
  -- -------------------------------------------------------------------------
  local function GetTextForPower()
    return "math.pow"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForRadians
  --
  -- This method returns the text for radians.
  -- -------------------------------------------------------------------------
  local function GetTextForRadians()
    return "math.rad"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForSine
  --
  -- This method returns the text for sine.
  -- -------------------------------------------------------------------------
  local function GetTextForSine()
    return "math.sin"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForHyperbolicSine
  --
  -- This method returns the text for hyperbolic sine.
  -- -------------------------------------------------------------------------
  local function GetTextForHyperbolicSine()
    return "math.sinh"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForSquareRoot
  --
  -- This method returns the text for square root.
  -- -------------------------------------------------------------------------
  local function GetTextForSquareRoot()
    return "math.sqrt"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForTangent
  --
  -- This method returns the text for tangent.
  -- -------------------------------------------------------------------------
  local function GetTextForTangent()
    return "math.tan"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForHyperbolicTangent
  --
  -- This method returns the text for hyperbolic tangent.
  -- -------------------------------------------------------------------------
  local function GetTextForHyperbolicTangent()
    return "math.tanh"
  end
  
  -- -------------------------------------------------------------------------
  -- Method GetTextForSlash
  --
  -- This method returns the text for '/'.
  -- -------------------------------------------------------------------------
  local function GetTextForSlash()
    return "/"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForColon
  --
  -- This method returns the text for ':'.
  -- -------------------------------------------------------------------------
  local function GetTextForColon()
    return ":"
  end

  -- -------------------------------------------------------------------------
  -- Method GetTextForZ
  --
  -- This method returns the text for 'Z'.
  -- -------------------------------------------------------------------------
  local function GetTextForZ()
    return "Z"
  end
  
  
  -- -------------------------------------------------------------------------
  -- Method Add
  --
  -- This method adds two numbers when there are two numbers to add.
  -- -------------------------------------------------------------------------
  local function Add(self)
    local numValue1
	  local numValue2
	  local numReturnValue
    local strArg2
    local strArg1
    local strReturnValue
	  local numStackSize
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (1 < numStackSize) then
	    numValue2 = self.tblStack:Pop()
	    numValue1 = self.tblStack:Pop()
	    strArg2   = string.format("%s", numValue2)
	    strArg1   = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
	    numReturnValue = numValue1 + numValue2

      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Subtract
  --
  -- This method subtracts two numbers when there are two numbers to subtract.
  -- -------------------------------------------------------------------------
  local function Subtract(self)
    local numValue1
	  local numValue2
	  local numReturnValue
	  local numStackSize
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()

	  if (1 < numStackSize) then
	    numValue2 = self.tblStack:Pop()
	    numValue1 = self.tblStack:Pop()
	    strArg2 = string.format("%s", numValue2)
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
	    numReturnValue = numValue1 - numValue2
    
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Multiply
  --
  -- This method multiplies two numbers when there are two numbers to multiply.
  -- -------------------------------------------------------------------------
  local function Multiply(self)
    local numValue1
	  local numValue2
	  local numReturnValue
	  local numStackSize
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()

	  if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
	    strArg2 = string.format("%s", numValue2)
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = numValue1 * numValue2
      
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
    end
	
	  self.strValue = ""
	  return strReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Divide
  --
  -- This method divides two numbers when there are two numbers to divide and
  -- when it is defined.
  -- -------------------------------------------------------------------------  
  local function Divide(self)
    local numValue1
	  local numValue2
	  local numReturnValue
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()

	  if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
	    strArg2   = string.format("%s", numValue2)
      
	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	  
	    if (0 == numValue2) then -- protect against division by zero
	      self.tblStack:Push(numValue2)
      else
        numValue1 = self.tblStack:Pop()
	      strArg1   = string.format("%s", numValue1)

	      if (true==IsDate(strArg1)) then
          numIsDate = 1
		      numValue1 = computeJulianDate(strArg1, true)
	      end
        
        numReturnValue = numValue1 / numValue2
        
        if (1==numIsDate) then
          strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
          self.tblStack:Push(strReturnValue)
        else    
	        self.tblStack:Push(numReturnValue)
          strReturnValue = string.format("%s", numReturnValue)
        end
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Power
  --
  -- This method raise a number to a power.
  -- -------------------------------------------------------------------------  
  local function Power(self)
    local numValue1
	  local numValue2
	  local numReturnValue
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()

	  if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()	  
      numValue1 = self.tblStack:Pop()
	    strArg2 = string.format("%s", numValue2)
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.pow(numValue1, numValue2)
            
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
    end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method AbsoluteValue
  --
  -- This method returns the absolute value of a given value.
  -- -------------------------------------------------------------------------  
  local function AbsoluteValue(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()

	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
      
      numReturnValue = math.abs(numValue1)

      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end      
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcCosine
  --
  -- This method returns the arccosine of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcCosine(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()

	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
      
      numReturnValue = math.acos(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcSine
  --
  -- This method returns the arcsine of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcSine(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()

	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
      
      numReturnValue = math.asin(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcTangent
  --
  -- This method returns the arctangent of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcTangent(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
      
      numReturnValue = math.atan(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method ArcTangent2
  --
  -- This method returns the arctangent of a given value.
  -- -------------------------------------------------------------------------  
  local function ArcTangent2(self)
    local numValue1
	  local numValue2
	  local numReturnValue
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (1 < numStackSize) then
      numValue2 = self.tblStack:Pop()
	    numValue1 = self.tblStack:Pop()
	    strArg2 = string.format("%s", numValue2)
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.atan2(numValue1, numValue2)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Ceiling
  --
  -- This method returns the ceiling.
  -- -------------------------------------------------------------------------  
  local function Ceiling(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.ceil(numValue1)

      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end      
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Cosine
  --
  -- This method returns the cosine.
  -- -------------------------------------------------------------------------  
  local function Cosine(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.cos(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method HyperbolicCosine
  --
  -- This method returns the hyperbolic cosine.
  -- -------------------------------------------------------------------------  
  local function HyperbolicCosine(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.cosh(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Degrees
  --
  -- This method returns the hyperbolic cosine.
  -- -------------------------------------------------------------------------  
  local function Degrees(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.deg(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Exp
  --
  -- This method returns the e to the given power.
  -- -------------------------------------------------------------------------  
  local function Exp(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.exp(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Floor
  --
  -- This method returns the floor of a given value.
  -- -------------------------------------------------------------------------  
  local function Floor(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.floor(numValue1)

      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end      
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method FMod
  --
  -- This method returns the fmod of a given value.
  -- -------------------------------------------------------------------------  
  local function FMod(self)
    local numValue1
	  local numValue2
	  local numReturnValue
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (1 < numStackSize) then
	    numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
	    strArg2 = string.format("%s", numValue2)
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.fmod(numValue1, numValue2)
            
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method NaturalLogarithm
  --
  -- This method returns the natural logarithm
  -- -------------------------------------------------------------------------  
  local function NaturalLogarithm(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.log(numValue1)
            
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Base10Logarithm
  --
  -- This method returns the base 10 logarithm
  -- -------------------------------------------------------------------------  
  local function Base10Logarithm(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.log10(numValue1)
            
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Maximum
  --
  -- This method returns the maximum
  -- -------------------------------------------------------------------------  
  local function Maximum(self)
    local numValue1
	  local numValue2
	  local numReturnValue
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (1 < numStackSize) then
	    numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
	    strArg2 = string.format("%s", numValue2)
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.max(numValue1, numValue2)
            
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Minimum
  --
  -- This method returns the minimum
  -- -------------------------------------------------------------------------  
  local function Minimum(self)
    local numValue1
	  local numValue2
	  local numReturnValue
    local strArg2
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (1 < numStackSize) then
	    numValue2 = self.tblStack:Pop()
      numValue1 = self.tblStack:Pop()
	    strArg2 = string.format("%s", numValue2)
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    numValue2 = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.min(numValue1, numValue2)
            
      if (1==numIsDate) then
        strReturnValue = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(numReturnValue, true))
        self.tblStack:Push(strReturnValue)
      else    
	      self.tblStack:Push(numReturnValue)
        strReturnValue = string.format("%s", numReturnValue)
      end
	  end
	
	  self.strValue = ""
	  return strReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Pi
  --
  -- This method returns pi
  -- -------------------------------------------------------------------------  
  local function Pi(self)
	  local numReturnValue
	
    numReturnValue = math.pi	
    self.tblStack:Push(numReturnValue)
	  self.strValue = ""
	  return numReturnValue
  end
  
  -- -------------------------------------------------------------------------
  -- Method Radians
  --
  -- This method returns the radians
  -- -------------------------------------------------------------------------  
  local function Radians(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.rad(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Sine
  --
  -- This method returns the sine
  -- -------------------------------------------------------------------------  
  local function Sine(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.sin(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method HyperbolicSine
  --
  -- This method returns the hyperbolic sine
  -- -------------------------------------------------------------------------  
  local function HyperbolicSine(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.sinh(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method SquareRoot
  --
  -- This method returns the square root
  -- -------------------------------------------------------------------------  
  local function SquareRoot(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.sqrt(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method Tangent
  --
  -- This method returns the tangent
  -- -------------------------------------------------------------------------  
  local function Tangent(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.tan(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end

  -- -------------------------------------------------------------------------
  -- Method HyperbolicTangent
  --
  -- This method returns the hyperbolic tangent
  -- -------------------------------------------------------------------------  
  local function HyperbolicTangent(self)
    local numValue1
	  local numReturnValue
    local strArg1
    local strReturnValue
    local numIsDate = 0
	
	  self:SetValue()
	  numStackSize = self.tblStack:Size()
	
	  if (0 < numStackSize) then
      numValue1 = self.tblStack:Pop()
	    strArg1 = string.format("%s", numValue1)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    numValue1 = computeJulianDate(strArg1, true)
	    end
	  	  
      numReturnValue = math.tanh(numValue1)
      self.tblStack:Push(numReturnValue)
	  end
	
	  self.strValue = ""
	  return numReturnValue
  end
  
  
  -- -------------------------------------------------------------------------
  -- Method Reset
  --
  -- This method resets the calculator to its initial state.
  -- -------------------------------------------------------------------------
  local function Reset(self)
    print("+Reset")
    local numItemCount
	  local numValue
	
    self.strValue = ""
	
	  numItemCount = self.tblStack:Size()
	
	  if (0 < numItemCount) then
      for numIndex = 1, numItemCount, 1
      do
        numValue = self.tblStack:Pop()
		    print(numValue)
      end
    end
	  print("-Reset")
  end
  

  -- -------------------------------------------------------------------------
  -- Method Evaluate
  --
  -- This method evaluates a postfix mathematical expression.
  -- -------------------------------------------------------------------------
local function Evaluate(a_strExpression)
  tblStack = {}  
  z = 0
  strText = ""
 
  local numYears1
  local numYears2
  local numDaysOfYear1
  local numDaysOfYear2
  local numHours1
  local numHours2
  local numMinutes1
  local numMinutes2
  local numSeconds1
  local numSeconds2
  local strArg1
  local strArg2
  local strYears1
  local strYears2
  local strDaysOfYear1
  local strDaysOfYear2
  local strHours1
  local strHours2
  local strMinutes1
  local strMinutes2
  local strSeconds1
  local strSeconds2
 
  --print(a_strExpression)
  for strToken in string.gmatch(a_strExpression, '%S+') do
    local numIsDate = 0
    
    if string.find(strToken, '^[-+*/]$')  then
      if 2>#tblStack then z = nil break end
      y       = table.remove(tblStack)
	    x       = table.remove(tblStack)
	    strArg2 = string.format("%s", y)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    y         = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
	  	  
      loadstring('z=x'..strToken..'y')()
      
      if (1==numIsDate) then
        z = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(z, true))
      end
    elseif string.find(strToken, "%^") then
      if 2>#tblStack then z = nil break end
      y = table.remove(tblStack)
	    x = table.remove(tblStack)
	    strArg2 = string.format("%s", y)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    y         = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.pow(x, y)
	  elseif string.find(strToken, "math.pow") then
      if 2>#tblStack then z = nil break end
      y = table.remove(tblStack)
	    x = table.remove(tblStack)
	    strArg2 = string.format("%s", y)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    y         = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.pow(x, y)
    elseif string.find(strToken, "math.abs") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.abs(x)
    elseif string.find(strToken, "math.acos") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.acos(x)
    elseif string.find(strToken, "math.asin") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.asin(x)
    elseif string.find(strToken, "math.atan2") then
	    if 2>#tblStack then z = nil break end
	    y = table.remove(tblStack)
	    x = table.remove(tblStack)
	    strArg2 = string.format("%s", y)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    y         = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.atan2(x,y)
    elseif string.find(strToken, "math.atan") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.atan(x)
    elseif string.find(strToken, "math.ceil") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.ceil(x)
      
      if (1==numIsDate) then
        z = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(z, true))
      end      
    elseif string.find(strToken, "math.cosh") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.cosh(x)
    elseif string.find(strToken, "math.cos") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.cos(x)
    elseif string.find(strToken, "math.deg") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.deg(x)
    elseif string.find(strToken, "math.exp") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.exp(x)
    elseif string.find(strToken, "math.floor") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.floor(x)
      
      if (1==numIsDate) then
        z = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(z, true))
      end
    elseif string.find(strToken, "math.fmod") then
	    if 2>#tblStack then z = nil break end
	    y = table.remove(tblStack)
	    x = table.remove(tblStack)
	    strArg2 = string.format("%s", y)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    y         = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.fmod(x,y)
    elseif string.find(strToken, "math.log10") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.log10(x)
    elseif string.find(strToken, "math.log") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.log(x)
    elseif string.find(strToken, "math.max") then
	    if 2>#tblStack then z = nil break end
	    y = table.remove(tblStack)
	    x = table.remove(tblStack)
	    strArg2 = string.format("%s", y)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    y         = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.max(x, y)
      
      if (1==numIsDate) then
        z = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(z, true))
      end      
    elseif string.find(strToken, "math.min") then
	    if 2>#tblStack then z = nil break end
	    y = table.remove(tblStack)
	    x = table.remove(tblStack)
	    strArg2 = string.format("%s", y)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg2)) then
        numIsDate = 1
		    y         = computeJulianDate(strArg2, true)
	    end
            
	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.min(x, y)
      
      if (1==numIsDate) then
        z = string.format("%04s/%03s/%02s:%02s:%02sZ", jd_to_cal(z, true))
      end      
    elseif string.find(strToken, "math.pi") then
	    z = math.pi
    elseif string.find(strToken, "math.rad") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.rad(x)
    elseif string.find(strToken, "math.sinh") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.sinh(x)
    elseif string.find(strToken, "math.sin") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.sin(x)
    elseif string.find(strToken, "math.sqrt") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.sqrt(x)
    elseif string.find(strToken, "math.tanh") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.tanh(x)
    elseif string.find(strToken, "math.tan") then
	    if 1>#tblStack then z = nil break end
	    x = table.remove(tblStack)
	    strArg1 = string.format("%s", x)

	    if (true==IsDate(strArg1)) then
        numIsDate = 1
		    x         = computeJulianDate(strArg1, true)
	    end
      
	    z = math.tan(x)
	  else
      if (true==IsDate(strToken)) then
        z = strToken
      else
        z = tonumber(strToken)  if nil==z then break end
      end
    end
    
    table.insert(tblStack, z)
  end
  
  numItemCount = #tblStack
  
  if 1==numItemCount and z then 
    strText = string.format("%s", z)
  elseif 1<numItemCount or nil==z then 
    strText = "error"
  end
  
  return strText
end

  -- -------------------------------------------------------------------------
  -- Method EvaluateInfix
  --
  -- This method evaluates an infix expression in Lua syntax.
  -- Note: This method does not handle dates arithmetic.
  -- -------------------------------------------------------------------------
  local function EvaluateInfix(a_strExpression)
    local fn
    local strText = ""
  
    if pcall(loadstring("return " .. a_strExpression)) then
      fn = loadstring("return " .. a_strExpression)
  
      if not fn or not fn() then
        strText = "error"
      else
        strText = string.format("%s", fn())
      end
    else
      strText = "error"
    end

    return strText
  end
    
  -- -------------------------------------------------------------------------
  -- Method GetType
  --
  -- This method returns the type of this class.
  -- -------------------------------------------------------------------------
  local function GetType()
    return "TModelCalculator"
  end

    local objGetTexts =
	{
	  GetTextForAbsoluteValue     = GetTextForAbsoluteValue,
	  GetTextForArcCosine         = GetTextForArcCosine,
	  GetTextForArcSine           = GetTextForArcSine,
	  GetTextForArcTangent        = GetTextForArcTangent,
	  GetTextForArcTangent2       = GetTextForArcTangent2,
	  GetTextForCeiling           = GetTextForCeiling,
	  GetTextForCosine            = GetTextForCosine,
	  GetTextForHyperbolicCosine  = GetTextForHyperbolicCosine,
	  GetTextForDegrees           = GetTextForDegrees,
	  GetTextForExp               = GetTextForExp,
	  GetTextForFloor             = GetTextForFloor,
	  GetTextForFMod              = GetTextForFMod,
	  GetTextForNaturalLogarithm  = GetTextForNaturalLogarithm,
	  GetTextForBase10Logarithm   = GetTextForBase10Logarithm,
	  GetTextForMaximum           = GetTextForMaximum,
	  GetTextForMinimum           = GetTextForMinimum,
	  GetTextForPi                = GetTextForPi,
	  GetTextForPower             = GetTextForPower,
	  GetTextForRadians           = GetTextForRadians,
	  GetTextForSine              = GetTextForSine,
	  GetTextForHyperbolicSine    = GetTextForHyperbolicSine,
	  GetTextForSquareRoot        = GetTextForSquareRoot,
	  GetTextForTangent           = GetTextForTangent,
	  GetTextForHyperbolicTangent = GetTextForHyperbolicTangent,
	  GetTextForSlash             = GetTextForSlash,
	  GetTextForColon             = GetTextForColon,
	  GetTextForZ                 = GetTextForZ,
	}

  -- -------------------------------------------------------------------------
  -- Method new_TModelCalculator
  --
  -- This method creates an object of this class.
  -- -------------------------------------------------------------------------
  function new_TModelCalculator()
	
    local objModelCalculator =
    {
      GetType                     = GetType,
	  Reset                       = Reset,
	  AppendCharZ                 = AppendCharZ,
	  AppendCharColon             = AppendCharColon,
	  AppendCharSlash             = AppendCharSlash,
	  AppendDecimalPoint          = AppendDecimalPoint,
	  AppendDigit0                = AppendDigit0,
	  AppendDigit1                = AppendDigit1,
	  AppendDigit2                = AppendDigit2,
	  AppendDigit3                = AppendDigit3,
	  AppendDigit4                = AppendDigit4,
	  AppendDigit5                = AppendDigit5,
	  AppendDigit6                = AppendDigit6,
	  AppendDigit7                = AppendDigit7,
	  AppendDigit8                = AppendDigit8,
	  AppendDigit9                = AppendDigit9,
      ToggleSign                  = ToggleSign,	  
	  GetPlusSign                 = GetPlusSign,
	  GetMinusSign                = GetMinusSign,
	  GetMultiplySign             = GetMultiplySign,
	  GetDivideSign               = GetDivideSign,
	  GetPowerSign                = GetPowerSign,
	  GetPlusMinus                = GetPlusMinus,
	  tblGetTexts                 = objGetTexts,
	  SetValue                    = SetValue,
	  Add                         = Add,
	  Subtract                    = Subtract,
	  Multiply                    = Multiply,
	  Divide                      = Divide,
	  Power                       = Power,
	  AbsoluteValue               = AbsoluteValue,
	  ArcCosine                   = ArcCosine,
	  ArcSine                     = ArcSine,
	  ArcTangent                  = ArcTangent,
	  ArcTangent2                 = ArcTangent2,
	  Ceiling                     = Ceiling,
	  Cosine                      = Cosine,
	  HyperbolicCosine            = HyperbolicCosine,
	  Degrees                     = Degrees,
	  Exp                         = Exp,
	  Floor                       = Floor,
	  FMod                        = FMod,
	  NaturalLogarithm            = NaturalLogarithm,
	  Base10Logarithm             = Base10Logarithm,
	  Maximum                     = Maximum,
	  Minimum                     = Minimum,
	  Pi                          = Pi,
	  Radians                     = Radians,
	  Sine                        = Sine,
	  HyperbolicSine              = HyperbolicSine,
	  SquareRoot                  = SquareRoot,
	  Tangent                     = Tangent,
	  HyperbolicTangent           = HyperbolicTangent,
	  Evaluate                    = Evaluate,
	  EvaluateInfix               = EvaluateInfix,
	  tblStack                    = new_TStack(),
	  strValue                    = "",
    }

    return objModelCalculator
  end
end  

